#include <proto.h>

#include "gtknob.h"
#include "gtscroller.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtKnob class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init knob.
*
\************************************************************************/
GtKnob *GtMkKnob(char *n,GtManager *p,GtTag *tgs)
{
	GtKnob *obj;
	
	if(!(obj=new GtKnob()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtKnob object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init knob.
*
\************************************************************************/
GtKnob *GtMkKnob(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtKnob *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkKnob(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty knob.
*
\************************************************************************/
GtKnob::GtKnob(void)
{
	down=FALSE;
	bdrw=1;
	minxs=5;
	minys=5;
}

/************************************************************************\
*
* Setup knob.
*
\************************************************************************/
sbool GtKnob::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_BDR,GTRT_BYTE,&bdrw);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Border:
				bdrw=t->d.val;
				break;
		}
	}
	this->SetMinMaxSize(2*bdrw+3,2*bdrw+3,GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set knob attributes.
*
\************************************************************************/
sbool GtKnob::Set(GtTag *tgs)
{
	GtTag *t;
	sbool bch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Border:
				bdrw=t->d.val;
				bch=TRUE;
				break;
		}
	}
	if(bch)
	{
		this->SetMinMaxSize(2*bdrw+3,2*bdrw+3,GTS_MAX,GTS_MAX);
		this->Refresh();
	}
	return(TRUE);
}

/************************************************************************\
*
* Get knob attributes.
*
\************************************************************************/
void GtKnob::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Border:
				*ptr=bdrw;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtKnob::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtKnob")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set knob event mask.
*
\************************************************************************/
void GtKnob::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask|ButtonPressMask|ButtonReleaseMask|
												Button1MotionMask);
}

/************************************************************************\
*
* Draw knob.
*
\************************************************************************/
sbool GtKnob::Draw(XEvent *xe)
{
	int n;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(down)
	{
		for(n=0; n<bdrw; n++)
		{
			GT3DBOX(gtdrk,gtlht,gtwin,n,n,xsize-2*n,ysize-2*n);
		}
	}
	else
	{
		for(n=0; n<bdrw; n++)
		{
			GT3DBOX(gtlht,gtdrk,gtwin,n,n,xsize-2*n,ysize-2*n);
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Mouse knob events.
*
\************************************************************************/
sbool GtKnob::Mouse(XEvent *xe)
{
	switch(xe->type)
	{
		case ButtonPress:
			if(xe->xbutton.button==Button1)
			{
				down=TRUE;
				xoff=xe->xbutton.x;
				yoff=xe->xbutton.y;
				dsp->SetFocus(this);
				this->Refresh();
				return(TRUE);
			}
			break;
		case ButtonRelease:
			if(xe->xbutton.button==Button1)
			{
				down=FALSE;
				dsp->ClrFocus(this);
				this->Refresh();
				parent->Notify(GT_Activate);
				return(TRUE);
			}
			break;
		case MotionNotify:
			if(down)
			{
				this->Move(xpos+(xe->xmotion.x-xoff),ypos+(xe->xmotion.y-yoff));
				return(TRUE);
			}
			break;
	}
	return(FALSE);
}
